#################################################################################
# Created       : Martin Foster
# Modified      : 09/03/2001
#################################################################################
#
# Comm - Module part of Webchat designed to prompt and handle posts.
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

package Ethereal::Comm;						# Pacakace name

use Ethereal::Date;						# Date hander
use Ethereal::Filter;						# Filter system
use Ethereal::Option;						# Options handling


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	#####################
	# Data Members
	my $class    = shift;					# Self reference
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI Handle

	my $self     = {
		HANDLE  => $database,
		CGI     => $cgi
 	 };


	#####################
	# Program Area

	# Bless $self into designated class
	bless $self, $class;

	# Return self
	return $self;
}



#####################
# Comm
#
# Subroutine primarily charged with displaying input area.   This will also call
# CommHandler which will handle the requests to determin the need for a new
# database entry

sub Comm
{
	#####################
	# Data members
	my $self     = shift;					# Self referencing link
	my $param    = shift;					# Parameter handle
	my $choice   = shift;					# Choice of the matter

	my $database = $self->{'HANDLE'};			# Database handle
	my $cgi      = $self->{'CGI'};				# CGI handle

	my $java;						# Java options

	my $inline;						# Inline argument list
	my $url      = $cgi->url();				# Url of script

	my $puppet;						# Puppet name
	my $color;						# Colour default
	my $face;						# Font face
	my $option;						# Option handle
	my $size;						# Size default

	my @color;						# Colours
	my @face;						# Font face
	my @size;						# Size fonts
	my @char;						# Character

	my %realm;						# Realm hash


	#####################
	# Program area

	# Link with Hash
	$database->GetHashRealm(\%realm);


	# Populate arrays
	$option = new Ethereal::Option(); 
	$option->Split(\@color, $realm{'OptTextColour'});
	$option->Split(\@size,  $realm{'OptTextSize'});
	$option->Split(\@face,  $realm{'OptTextFace'});

	# Populate puppet list
	$database->GetListPuppetYours(\@char, $param->{'USER'});
	

	# Cleanup for embed
	$param->Cleanup('POST');


	# Define inline and puppet
	$inline = $param->EmbedInline($cgi, %{$param});
	$puppet = $param->{'CHAR'};

	# Retreive size and color
	($size, $color, $face) = $database->DataGetFont($puppet, $color[0], $face[0]);

	
	# Independent HTML
	if ($choice != 0)
	{

		# Print file and HTML header
		print $cgi->header(-target=>'Comm');
		$database->DocumentGetHeaderPlain();
	}

	# With or without javascript as prescribed
	# Determine need for javascript
	($java) = $database->DataGetJavascript($param->{'USER'});

	# Print if javascript is desired
	if ($java eq 'yes')
	{ $database->DocumentGetJavascript($Database); }

	# HTML
	# Start form
	print $cgi->start_form();

	# Embed Main or Comm
	# For basic chat
	if ($choice == 0)
	{ print $param->EmbedNormal($cgi, %{$param}), "\n"; }

	# Other modes
	else
	{ print $param->EmbedNormal($cgi, %{$param}, 'COMM', 'true'), "\n"; }


	# Top panel and links
	if ($java eq 'yes')
	{
		# Top of table
		print "<CENTER>\n<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=2 BGCOLOR=\"$realm{'SetBGColor'}\" WIDTH=505>\n";

		# Javascript enabled
		print "\t<TR>\n\t\t<TD COLSPAN=\"4\">",
		     "$realm{'SetTop'}",
 		     "<A HREF=\"javascript:popup_window('$realm{'LnkPuppet'}$inline',1)\">$realm{'TagLnkPuppet'}</A>", 
		     "<A HREF=\"javascript:popup_window('$realm{'LnkIgnore'}$inline',1)\">$realm{'TagLnkIgnore'}</A>",
		     "<A HREF=\"javascript:popup_window('$realm{'LnkWho'}$inline',1)\">$realm{'TagLnkWho'}</A>",
		     "<A HREF=\"javascript:popup_window('$realm{'LnkPrivate'}$inline',1)\">$realm{'TagLnkPrivate'}</A>",
		     "<A HREF=\"javascript:popup_window('$realm{'LnkRoll'}$inline',1)\">$realm{'TagLnkRoll'}</A>",
	     	     "</TD>\n\t</TR>\n"; 
	}
	else
	{
		# Top of table
		print "<CENTER>\n<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=2 BGCOLOR=\"$realm{'SetBGColor'}\" WIDTH=505>\n";

		# Self containted window
		print "\t<TR>\n\t\t<TD COLSPAN=\"4\">",
		      "$realm{'SetTop'}",
		      "<A HREF=\"$realm{'LnkPuppet'}$inline\" TARGET=\"_blank\">$realm{'TagLnkPuppet'}</A>", 
		      "<A HREF=\"$realm{'LnkIgnore'}$inline\" TARGET=\"_blank\">$realm{'TagLnkIgnore'}</A>",
		      "<A HREF=\"$realm{'LnkWho'}$inline\" TARGET=\"_blank\">$realm{'TagLnkWho'}</A>", 
		      "<A HREF=\"$realm{'LnkPrivate'}$inline\" TARGET=\"_blank\">$realm{'TagLnkPrivate'}</A>",
		      "<A HREF=\"$realm{'LnkRoll'}$inline\" TARGET=\"_blank\">$realm{'TagLnkRoll'}</A>",
		      "</TD>\n\t</TR>\n"; 
	}


	# Posting field and side spacers
	print "\t<TR>\n\t\t<TD ROWSPAN=\"7\" WIDTH=\"5\"><FONT COLOR=\"$realm{'SetBGColor'}\">&nbsp;</FONT></TD>\n";
	print "\t\t<TD><TEXTAREA NAME=\"POST\", ROWS=\"3\", COLS=\"50\", WRAP=\"SOFT\"></TEXTAREA></TD>\n";
	print "\t\t<TD>", $cgi->submit($realm{'TxtSubmitButton'}) ,"</TD>\n";
	print "\t\t<TD ROWSPAN=\"7\" WIDTH=\"5\"><FONT COLOR=\"$realm{'SetBGColor'}\">&nbsp;</FONT></TD>\n\t</TR>\n";

	# For basic and framed 
	if ($choice < 1) 
	{
		print "\t<TR>\t\t<TD COLSPAN=\"2\" ALIGN=\"CENTER\"> ", 
		      $cgi->checkbox(-name=>'SCROLLBACK', -label=>$realm{'TxtScrollback'}),
		      " </TD>\n\t</TR>\n";
	}

	# Framed and Streaming modes
	else
	{
		# Spacing
		print "\t<TR>\n\t\t<TD COLSPAN=\"2\" HEIGHT=\"6\">&nbsp;</TD>\n\t</TR>\n";
	}

	# Spacing
	print "\t<TR>\n\t\t<TD COLSPAN=\"2\" HEIGHT=\"6\">&nbsp;</TD>\n\t</TR>\n";

	# Puppet selection
	print "\t<TR>\n\t\t<TD COLSPAN=\"2\">$realm{'TagPuppet'}", $cgi->popup_menu('CHAR', \@char, $puppet), "</TD>\n\t</TR>\n";

	# Text modifications
	print "\t<TR>\n\t\t<TD COLSPAN=\"2\">$realm{'TagTextColour'}",  $cgi->popup_menu('COLOR', \@color, $color), "</TD>\n\t</TR>\n";
	print "\t<TR>\n\t\t<TD COLSPAN=\"2\">$realm{'TagTextFace'}", $cgi->popup_menu('FACE', \@face, $face), "</TD>\n\t</TR>\n";
	print "\t<TR>\n\t\t<TD COLSPAN=\"2\">$realm{'TagTextSize'}", $cgi->popup_menu('SIZE', \@size, $size), "</TD>\n\t</TR>\n";


	# Bottom bar
	# Re-Assign Inline
	$inline = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'});

	# Javascript
	if ($java eq 'yes')
	{
		# Show link
		print "\t<TR>\n\t\t<TD COLSPAN=\"4\">",
		      "<A HREF=\"$realm{'LnkPortal'}$inline\" TARGET=\"_blank\">$realm{'TagLnkPortal'}</A>",
		      "<A HREF=\"javascript:popup_window('$realm{'LnkBoard'}$inline',1)\">$realm{'TagLnkBoard'}</A>",
		      "<A HREF=\"javascript:popup_window('$realm{'LnkMail'}$inline&LIST=True',1)\">$realm{'TagLnkMail'}</A>",
		      "</TD>\n\t</TR>\n"; 
	}
	# Lack thereof
	else
	{
		# Show link
		print "\t<TR>\n\t\t<TD COLSPAN=\"4\">",
		      "<A HREF=\"$realm{'LnkPortal'}$inline\" TARGET=\"_blank\">$realm{'TagLnkPortal'}</A>",
		      "<A HREF=\"$realm{'LnkBoard'}$inline\" TARGET=\"_blank\">$realm{'TagLnkBoard'}</A>",
		      "<A HREF=\"$realm{'LnkMail'}$inline&LIST=True\" TARGET=\"_blank\">$realm{'TagLnkMail'}</A>",
		      "</TD>\n\t</TR>\n"; 
	}


	# Table Bottom
	print "</TABLE>\n</CENTER>\n";

	# End of form
	print $cgi->end_form(), "\n";

	# Print end of html if needed
	if ($choice != 0)
	{ print $cgi->end_html(); }
}

# Alias for dependency
sub CommDependant
{ Comm(shift, shift, 0); }

# Alias for independnt framed
sub CommIndependant
{ Comm(shift, shift, 1); }

# Alias for independant streaming
sub CommStream
{ Comm(shift, shift, 2); }

# Alias for independant auto-scrolling streaming
sub CommAutoScroll
{ Comm(shift, shift, 3); }


#####################
# DataWrite
#
# Encharged with properly handling information and actually making posts
# once formatted inserting them into the database.

sub DataWrite							# Database, param
{
        #####################
        # Data members
	my $self     = shift;					# Self referencing link
	my $param    = shift;					# Parameter handle

	my $database = $self->{'HANDLE'};			# Database handle
	my $data     = $database->{'HANDLE'};			# CGI handle

	my $option = new Ethereal::Option();			# Option handler

	my $hex;						# Hex colour

	my %realm;						# Realm hash


        #####################
        # Data members

	# Assign to target if undefined
	$param->{'TARGET'} = (defined($param->{'TARGET'})) ? $param->{'TARGET'} : 'all';

	# Only do following if posts exist
	if ((defined($param->{'POST'})) && ((length($param->{'POST'}) > 1)))
	{
		# Link DBM to hash
		$database->GetHashRealm(\%realm);

		# Determin font information
		$hex   = $option->Find($realm{'OptTextColour'}, $realm{'OptTextColourHex'}, $param->{'COLOR'});

		# Update if not a dice roll or system wide post
		if (($param->{'TARGET'} ne 'dice roll') && ($param->{'TARGET'} ne $realm{'TxtSystem'}))
		{
			# Update database entry
			$data->do("UPDATE Puppet 
				SET PuppetTextColour=?, 
				    PuppetTextSize=?,
				    PuppetTextFace=? 
				WHERE PuppetName=?", {}, 

			 $param->{'COLOR'}, 
		 	 $param->{'SIZE'},
			 $param->{'FACE'}, 
			 $param->{'CHAR'});
		}

		# Handle posts
		$self->PostHandler($param, \%realm); 		
	}
	
	# No post simply post for timestamp purposes
	else
	{
		$data->do("INSERT INTO Post 
			(PostIDNumber,
			 RealmName, 
			 PuppetName,
			 PostTo,
			 PostTimeStamp)
			VALUES(NULL, ?, ?, 'system', ?)", {}, 

		 $param->{'ROOM'}, 
		 $param->{'CHAR'}, 
		 time);
	}
}

####################
# Gummy Handler
#
# This will control how a gummy is shown.   This will vary depending on access
# and which real they access at the time.  Since such behavior is for 
# moderators or supervisors only this will not be triggered often.

sub GummyHandler
{
	#####################
	# Data members
	my $self     = shift;					# Self referencing link
	my $database = shift;					# Database handle
	my $param    = shift;					# Parameter handle
	my $option   = shift;					# Option handler
	my $realm    = shift;					# Realm hash
	my $gummy    = shift;					# Gummy handling

	my $gummies;						# Singular gummy
	my @gummies;						# Array of gummies

	my %config;						# Configuration handle


	#####################
	# Program Area

	# Only if defined
	if (defined($gummy))
	{

		# Retreive privs
		my ($super, $admin, $mod) = $database->DataGetPrivs($param->{'USER'});

		# Only if supervisory or moderator
		if (($super eq 'yes') || ($mod eq 'yes'))
		{
			# Retreive access
			my ($choice) = $database->DataGet("SELECT PuppeteerLogin
				FROM  RealmSupervisor
				WHERE RealmName=?
				  AND PuppeteerLogin=?", 

			 $param->{'ROOM'},
			 $param->{'USER'});

			# If in a restricted realm
			unless (defined($choice))
			{
				# Connect to hash
				$database->GetHashConfiguration(\%config);

				# Retreive gummies
				$option->Split(\@gummies, "$config{OptGummySupFiles}:$config{OptGummyModFiles}");		


				# Cycle through
				foreach $gummies (@gummies)
				{
					# If gummy is restricted
					if ($gummies eq $gummy)
					{
						# Force as default
						$gummy = $realm->{OptGummyDefault};

						# Leave loop
						last; 
					}
				}
			}
		}
	}
		
	# Gummy assignment
	$gummy = (defined($gummy)) ? "$realm->{SetGummyRoot}$gummy" : "$realm->{SetGummyRoot}$realm->{OptGummyDefault}";  

	# Return
	return $gummy;
}


#####################
# Post Handler
#
# Depending on rules will handle the formatting of the three styles of posts
# filter language and HTML if needed or specified.

sub PostHandler
{
	#####################
	# Data members
	my $self     = shift;					# Self referencing link
	my $param    = shift;					# Parameter handle
	my $realm    = shift;					# Reference to realm

	my $cgi      = $self->{'CGI'};				# CGI handle

	my $database = $self->{'HANDLE'};			# Database handle

	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $p_handle;						# Puppet information
	my $r_handle;						# Realm information

	my $date     = new Ethereal::Date();			# Date handler
	my $filter   = new Ethereal::Filter();			# Filter handler
	my $option   = new Ethereal::Option();			# Option handler

	my $p_res;						# Puppet results
	my $r_res;						# Realm results

	my $full;						# Full formating
	my $imageless;						# No images
	my $spartan;						# Spartan formatting

	my $bio;						# Bio link
	my $gummy;						# Gummy link
	my $image;						# Image section
	my $inline;						# Inline parameters
	my $privs;						# Room privs
	my $tabletop;						# Table top 
	my $tag;						# Tagline
	my $title;						# Title

	my $post;						# Actual post
	my $position = 0;					# Position of post

	my $start;						# Start of table
	my $end;						# End of table

	my $time     = $date->GetDate(time);			# Date and Time
	my $target   = $param->{'TARGET'};			# Target to

	my @lang;						# Language array
	my @posts;						# Array of posts
	my @realmpic;						# Ream picture restriction


	#####################
	# Program area

	# Retreive puppet and realminformation
	# Puppet information
	$p_handle = $data->prepare("SELECT 
		  PuppetName,
		  PuppeteerLogin, 
		  PuppetTag,
		  PuppetPic,
		  PuppetPicHeight,
		  PuppetPicWidth,
		  PuppetTitle, 
		  PuppetGummy 
		FROM  Puppet 
		WHERE PuppetName=?");

	# Realm Information
	$r_handle  = $data->prepare("SELECT 
		  RealmLanguage, 
		  RealmTags 
		FROM  Realm 
		WHERE RealmName=?");

	# Execute
	$p_handle->execute($param->{'CHAR'});
	$r_handle->execute($param->{'ROOM'});
	
	# Associate to hash
	$p_res = $p_handle->fetchrow_hashref();
	$r_res = $r_handle->fetchrow_hashref(); 	


	# Determine colour to use in frames
	if ($target eq 'all')
	{ $tabletop = "WIDTH=\"100%\""; }
	elsif ($target eq $realm->{'TxtSystem'})
	{ $tabletop = "BORDER=\"$realm->{SetBorderSize}\" WIDTH=\"100%\">\n\t<TR BGCOLOR=\"$realm->{SetBackSystem}\">\n\t\t<TD>$realm->{TagHeadSystem}</TD>\n\t<TR"; }
	elsif ($target eq 'dice roll')
	{ $tabletop = "BORDER=\"$realm->{SetBorderSize}\" WIDTH=\"100%\">\n\t<TR BGCOLOR=\"$realm->{SetBackDice}\">\n\t\t<TD>$realm->{TagHeadDice}</TD>\n\t<TR"; }
	elsif ($target ne 'system')
	{ $tabletop = "BORDER=\"$realm->{SetBorderSize}\" WIDTH=\"100%\">\n\t<TR BGCOLOR=\"$realm->{SetBackPrivate}\">\n\t\t<TD>$realm->{TagHeadPrivate}</TD>\n\t<TR"; }

	# Language filtering
	if ($r_res->{'RealmLanguage'} eq 'restricted')
	{
		# Retreive words
		$database->GetListWords(\@lang, $param->{'ROOM'});

		# Run through
		my $word;

		while ($word = shift(@lang))
		{
			# Run a global replace
			$param->{'POST'} =~ s/$word/****/gsi;
		}
	}

	# Bio link
	# Determin gummy
	$gummy = $self->GummyHandler($database, $param, $option, $realm, $p_res->{'PuppetGummy'});


	# $gummy = (defined($p_res->{'PuppetGummy'})) ? "$realm->{SetGummyRoot}$p_res->{'PuppetGummy'}" : "$realm->{SetGummyRoot}$realm->{OptGummyDefault}";  


	# Inline and format
	$inline = $param->EmbedInline($cgi, 'CHAR', $param->{'CHAR'});
	$bio = "<A HREF=\"$realm->{'LnkBio'}$inline\" TARGET=\"_blank\"><IMG SRC=\"$gummy\" BORDER=\"0\" ALIGN=\"LEFT\"></A>";

	# Format title
	if (defined($p_res->{'PuppetTitle'}))
	{ 
		# Verify if NAME attribute exists
		if ($p_res->{'PuppetTitle'} =~ /NAME/)
		{
			$p_res->{'PuppetTitle'} =~ s/NAME/$param->{'CHAR'}/g;
			$title = $p_res->{'PuppetTitle'};
		}

		# Otherwise prepend name
		else
		{ $title = "$param->{CHAR} $p_res->{PuppetTitle}"; }
	}
	

	# Assign chacter name to title
	else
	{ $title = $param->{'CHAR'}; }	

	# Create formatted line
	$title = "$bio <B><FONT SIZE=\"$realm->{'SetTitleSize'}\">$title</FONT></B>";


	# Tagline handling
	$tag = (defined($p_res->{'PuppetTag'})) ? "(<I>$puppet$p_res->{'PuppetTag'}</I>)" : "";

	# Create tagline
	unless (($target ne 'all') 
            &&  ($target ne 'system')
	    &&  ($target ne 'dice roll')
	    &&  ($target ne $realm->{'TxtSystem'}))
	{ $tag   = "$tag ($time)"; }
	else
	{ $tag   = "$tag ($realm->{'TagPuppetPMFrom'}: $param->{'CHAR'}) ($realm->{'TagPuppetPMTo'}: $param->{'OTARGET'}) ($time)"; }
		

	# Determine colour
	my $hex = $option->Find($realm->{'OptTextColour'}, $realm->{'OptTextColourHex'}, $param->{'COLOR'});

	# Image handling
	$image = (defined($p_res->{'PuppetPic'})) ?  "<A HREF=\"$p_res->{'PuppetPic'}\" TARGET=\"_TOP\">$realm->{'TagLnkImage'}</A><BR>" : ""; 


	# Filtering of the text
	# Filter HTML if option is selected
	if ($r_res->{'RealmTags'} eq 'restricted')
	{ $param->{'POST'} =~ s/<[^>]*>//gs; }

	# Standard filtering
	else
	{ ($param->{'POST'}) = $filter->SafeHTML($param->{'POST'}); }


	# Large post handling
	# Divide and conquer
	if ((length($param->{'POST'})) > 5000)
	{
		# Cannot allow stray HTML tags
		$param->{'POST'} =~ s/<[^>]*>//gs;

		# Prepend beginning of posts
		$post = substr($param->{'POST'}, 0, 5000);
		push(@posts, "$post ...");

		$position++;

		# Truncate into edible bites
		while (length(substr($param->{'POST'}, ($position * 5000), 5001)) > 5000)
		{
			$post = substr($param->{'POST'}, ($position * 5000), 5000);
			push(@posts, "... $post ...");

			$position++;
		}

		# Append whatever is left
		$post = substr($param->{'POST'}, ($position * 5000), 5000);
		push(@posts, "... $post");
	}
	# Pass as first element of array
	else
	{ push(@posts, "$param->{'POST'}"); }


	# Clear post
	$param->Cleanup('POST');


	# Cycle though individual sections
	foreach $post (@posts)
	{

		# Handling of links
		$post      =~ s/$/\n/;
		$post      =~ s/(http:\/\/.*?)[\s]/<A HREF="$1" TARGET="_blank">$1<\/A> /gm;

		# Add colors
		$post = "<FONT SIZE=\"$param->{'SIZE'}\" COLOR=\"$hex\" FACE=\"$param->{'FACE'}\">$post</FONT>";

		# Full formatting
		if (defined($p_res->{'PuppetPic'}))
		{
			# Retreive image size information
			my ($p_height, $p_width) = $database->DataGetRealmImageInfo($param->{'ROOM'});
		
			# Format image format accordingly
			if ((($p_height != 0) 
			 && ($p_width != 0)) 
			 && (($p_res->{'PuppetPicHeight'} > $p_height) 
			 || ($p_res->{'PuppetPicWidth'} > $p_width))) 
			{ $full = "$image$title $tag<BR><BR>$post"; }
			else
			{ $full = "<IMG SRC=\"$p_res->{'PuppetPic'}\" HEIGHT=\"$p_res->{'PuppetPicHeight'}\" WIDTH=\"$p_res->{'PuppetPicWidth'}\" ALIGN=\"left\" BORDER=\"0\">$title $tag<BR><BR>$post"; }
		}
		else
		{ $full = "$title $tag<BR><BR>$post"; }
	

		# Format non imaged format
		$imageless = "$image$title $tag<BR><BR>$post";


		# Create spartan format
		$post =~ s/<[^>]*>//gs;
		$spartan = "$image$title $tag<BR><BR>$post<BR><BR>";


		# Embed all formats in tables
		$start = "<BR>\n<TABLE $tabletop>\n\t<TR>\n\t\t<TD>";
		$end   = "</TD>\n\t</TR>\n</TABLE>\n";

		# Convert \n to a useable character	
		$full      =~ s/\n/<BR>/gs;
		$imageless =~ s/\n/<BR>/gs;
		$spartan   =~ s/\n/<BR>/gs;

		# Assembly
		$full      = "$start$full$end";
		$imageless = "$start$imageless$end";
		$spartan   = "$start$spartan$end";


		# Normal messages
		if (($target eq 'all') || ($target eq 'dice roll'))
		{
			# Insert into database
			$data->do("INSERT INTO Post 
				(PostIDNumber,
				 RealmName,
				 PuppetName,
				 PostTo,
				 PostTimestamp,
				 PostFullFormat,
				 PostImagelessFormat,
				 PostPartialFormat)		 
				VALUES(NULL, ?, ?, ?, ?, ?, ?, ?)", {}, 
	
			 $param->{'ROOM'}, 
			 $param->{'CHAR'}, 
			 'all', 
			 time, 
			 $full, 
			 $imageless, 
			 $spartan);
		}

		# System wide
		elsif ($target eq $realm->{'TxtSystem'})
		{
			#####################
			# Data Members

			my @realms;					# List of realms
			my $realm;					# Specific realm

			# Retreive lists of realms
			$database->GetListGlobalRealms(\@realms);

			# Run through each realm
			foreach $realm (@realms)
			{
				# Insert into database
				$data->do("INSERT INTO Post 
					(PostIDNumber,
					 RealmName,
					 PuppetName,
					 PostTo,
					 PostTimestamp,
					 PostFullFormat,
					 PostImagelessFormat,
					 PostPartialFormat)
					VALUES(NULL, ?, ?, ?, ?, ?, ?, ?)", {}, 
				 $realm, 
				 $param->{'CHAR'}, 
				 'all', 
				 time, 
				 $full, 
				 $imageless, 
				 $spartan);
			}
		}

		# Insert message to be returned to poster
		elsif ($param->{'TARGET'} ne 'system')
		{
			# Send to target
			$data->do("INSERT INTO Post 
				(PostIDNumber,
				 RealmName,
				 PuppetName,
				 PostTo,
				 PostTimestamp,
				 PostFullFormat,
				 PostImagelessFormat,
				 PostPartialFormat)
				VALUES(NULL, ?, ?, ?, ?, ?, ?, ?)", {}, 
			 $param->{'ROOM'}, 
			 $param->{'CHAR'}, 
			 $param->{'TARGET'}, 
			 time, 
			 $full, 
			 $imageless, 
			 $spartan);

			# Send back owner
			$data->do("INSERT INTO Post 
				(PostIDNumber,
				 RealmName,
				 PuppetName,
				 PostTo,
				 PostTimestamp,
				 PostFullFormat,
				 PostImagelessFormat,
				 PostPartialFormat)
				VALUES(NULL, ?, ?, ?, ?, ?, ?, ?)", {}, 
			 $param->{'ROOM'}, 
			 $param->{'CHAR'}, 
			 $param->{'SOURCE'}, 
			 time, 
			 $full, 
			 $imageless, 
			 $spartan);
		}
	}


	# End of handles
	$r_handle->finish();
	$p_handle->finish();
}

1;								# Return true
