#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 09/05/2001
#################################################################################
#
# Realm - Script part of Webchat designed to use the realms
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Comm;						# Communication handler
use Ethereal::Database;						# Database handler
use Ethereal::Login;						# Login functionality
use Ethereal::Param;						# Parameter control
use Ethereal::Post;						# Post handler
use Ethereal::Who;						# Who is on



#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $comm;							# Communications panel handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle
my $post;							# Post handle

my $chatpref;							# Chat preferences
my $inline;							# Inline parameters
my $javascript;							# Javascript preferences
my $url;							# URL path

my %realm;							# Realm hash

#################################################################################
# Program Area
#################################################################################

	# Initial handle
	$cgi      = new CGI;	
	$database = new Ethereal::Database();
	$param    = new Ethereal::Param();

	# Connect and fetch
	$database->Connect();
	$param->GetParam($cgi);

	# Fetch hash
	$database->GetHashRealm(\%realm);

	# Fetch URL
	$url = $cgi->url();

	
	# Login handle
	$login = new Ethereal::Login($database, $cgi, $param);

	# Authenthication
	if ($login->GetVerificationRealm())
	{
		# Make sure logins are possible
		$login->CheckLegality(\%realm);

		# Realm specific handles
		$comm  = new Ethereal::Comm($database, $cgi);
		$post  = new Ethereal::Post();
	

		# Retreive javascript and chat preferences
		($javascript, $chatpref) = $database->DataGetChatOptions($param->{'USER'});

		# Handle post writing
		$comm->DataWrite($param);

		# Framed chat
		if ($chatpref eq 'framed')
		{
			# Only do following if called with no parameters
			unless ((defined($param->{'MAIN'})) || (defined($param->{'COMM'})))
			{
				# Retreive parameter list
				$inline = $param->EmbedInline($cgi, %{$param});


				# Print file header
				print $cgi->header();

				# Print header
				print "<HTML>\n<HEAD>\n\t<TITLE>$param->{'ROOM'}</TITLE>\n</HEAD>\n\n";

				# Print frameset
				print "<FRAMESET ROWS=\"*,115\">\n";
				print "\t<FRAME NAME=\"Main\" SRC=\"$url$inline&MAIN=True\">\n";
				print "\t<FRAME NAME=\"Comm\" SRC=\"$url$inline&COMM=True\">\n";
				print "</FRAMESET>\n";

				# End of HTML
				print "</HTML>\n";
			}
	
			# Message display area
			if (defined($param->{'MAIN'}))
			{ 
				# File and HTML header
				print $cgi->header(-target=>'Main');
				$database->DocumentGetHeaderPlain();

				# Posting related
				$post->HandlePosts($database, $param); 

				# Parameter handler
				$param->Cleanup('MAIN');


				# Activation button
				print $cgi->start_form();
				print $param->EmbedNormal($cgi, %{$param}, 'MAIN', 'True'), "\n";
				
				# Table information
				print "<BR>\n<CENTER><TABLE WIDTH=\"500\">\n",
				      "\t<TR>\n",
				      "\t\t<TD>", $cgi->checkbox(-name=>'SCROLLBACK', -label=>$realm{'TxtScrollback'}), "</TD>\n",
				      "\t\t<TD>", $cgi->submit($realm{'SetPost'}), "</TD>\n",
				      "\t</TR>\n",
				      "\t</TABLE></CENTER>\n";

				print $cgi->end_form(), "\n";
				print $cgi->end_html();
			}

			# Comm panel
			if (defined($param->{'COMM'}))
			{ 
				# Cleanup
				$param->Cleanup('COMM');

				# Communications panel
				$comm->CommIndependant($param); 
			}
		}


		# Streaming chat with autoscroll
		elsif (($chatpref eq 'autoscroll') && ($javascript eq 'yes')) 
		{
			# Only do following if called with no parameters
			unless ((defined($param->{'MAIN'})) || (defined($param->{'COMM'})))
			{
				# Retreive parameter list
				$inline = $param->EmbedInline($cgi, %{$param});


				# Print file header
				print $cgi->header();				
				
				# Print header
				print "<HTML>\n<HEAD>\n\t<TITLE>$param->{'ROOM'}</TITLE>\n</HEAD>\n\n";

				# Print frameset
				print "<FRAMESET COLS=\"65,*\">\n";
				print "\t<FRAME NAME=\"Remote\" SRC=\"$realm{'LnkRemote'}\">\n";
				print "\t<FRAMESET ROWS=\"*,115\">\n";
				print "\t\t<FRAME NAME=\"Main\" SRC=\"$url$inline&MAIN=aff\">\n";
				print "\t\t<FRAME NAME=\"Comm\" SRC=\"$url$inline&COMM=aff\">\n";
				print "\t</FRAMESET>\n";
				print "</FRAMESET>\n";

				# End of HTML
				print "</HTML>\n";
			}	
	
			# Message display area
			if (defined($param->{'MAIN'}))
			{ 
				# Flush standard output				
				$| = 1;
				print "";

				# File header
				print $cgi->header(-target=>'Main');

				# Java enabled post window
				$database->DocumentGetHeaderPlain();
				$post->HandleStream($database, $cgi, $param); 
				print $cgi->end_html();
			}

			# Comm panel
			if (defined($param->{'COMM'}))
			{
				$comm->CommAutoScroll($param);
			}
		}

		# Streaming chat
		elsif (($chatpref eq 'stream') || ($chatpref eq 'autoscroll'))
		{
			# Only do following if called with no parameters
			unless ((defined($param->{'MAIN'})) || (defined($param->{'COMM'})))
			{
				# Retreive parameter list
				$inline = $param->EmbedInline($cgi, %{$param});


				# Print file header
				print $cgi->header();				
				
				# Print header
				print "<HTML>\n<HEAD>\n\t<TITLE>$param->{'ROOM'}</TITLE>\n</HEAD>\n\n";

				# Print frameset
				print "<FRAMESET ROWS=\"*,115\">\n";
				print "\t<FRAME NAME=\"Main\" SRC=\"$url$inline&MAIN=True\">\n";
				print "\t<FRAME NAME=\"Comm\" SRC=\"$url$inline&COMM=True\">\n";
				print "</FRAMESET>";

				# End of HTML
				print "</HTML>\n";
			}	
	
			# Message display area
			if (defined($param->{'MAIN'}))
			{ 
				# Flush standard output				
				$| = 1;
				print "";

				# File header
				print $cgi->header(-target=>'Main');

				# Posting related
				$database->DocumentGetHeaderPlain();
				$post->HandleStream($database, $cgi, $param); 
				print $cgi->end_html();
			}

			# Comm panel
			if (defined($param->{'COMM'}))
			{ 
				$comm->CommStream($param); 
			}
		}


		# Basic chat system
		else
		{
			# Print file and HTML header
			print $cgi->header();
			$database->DocumentGetHeaderPlain();

			# Message display and posting
			$post->HandlePosts($database, $param); 

			# Spacer
			print "<BR><BR>\n";

			$comm->CommDependant($param);

			# HTML footer
			print $cgi->end_html();
		}
	}

